const copyToClipboard = (s) => {
  document.addEventListener('copy', (e) => {
    e.preventDefault();
    e.clipboardData?.setData('text/plain', s);
    document.removeEventListener('copy', listener);
  });

  if (!navigator.clipboard) {
    document.execCommand('copy');
  } else {
    navigator.clipboard.writeText(s).catch(() => {
      console.log('failed to copy');
    });
  }
};

const buildGuesses = () => {
  const rows = document
    .querySelector('game-app')
    .shadowRoot.querySelector('game-theme-manager')
    .querySelectorAll('game-row');
  const isDarkMode = window.localStorage.darkTheme === 'true';
  return [...rows]
    .map((row) => {
      const guess = row.attributes.letters.value;

      if (!guess) {
        return null;
      }

      const colors = [...row.shadowRoot.querySelectorAll('game-tile')]
        .map((letter) => {
          switch (letter.attributes.evaluation.value) {
            case 'present':
              return '🟨';
            case 'correct':
              return '🟩';
            default:
              return isDarkMode ? '⬛' : '⬜';
          }
        })
        .join('');

      return `${colors} [spoiler]${guess.toUpperCase()}[/spoiler]`;
    })
    .filter((el) => !!el);
};

const buildShareCopy = () => {
  const app = new window.wrappedJSObject.wordle.bundle.GameApp();
  const number = app.dayOffset;
  const hardMode = app.isHardMode ? '*' : '';
  const guesses = buildGuesses();
  const guessesCopy = guesses.join('\n');

  return `Wordle ${number} ${guesses.length}/6${hardMode}\n\n${guessesCopy}`;
};

const handleClick = (e) => {
  e.preventDefault();
  e.stopPropagation();

  const shareCopy = buildShareCopy();
  const toastContainer = document
    .querySelector('game-app')
    .shadowRoot.querySelector('game-theme-manager')
    .querySelector('#system-toaster');
  const toast = document.createElement('game-toast');
  toast.setAttribute('text', 'Copied results to clipboard');
  toast.setAttribute('duration', '2000');
  toastContainer.appendChild(toast);
  copyToClipboard(shareCopy);
};

const buildButton = () => {
  const button = document.createElement('button');
  button.id = 'extended-share-button';
  button.innerText = 'Share ++';
  button.addEventListener('click', handleClick);
  button.style.cssText = `
    align-items: center;
    background-color: var(--key-bg-correct);
    border-radius: 4px;
    border: none;
    color: var(--key-evaluated-text-color);
    cursor: pointer;
    display: flex;
    filter: brightness(100%);
    font-family: inherit;
    font-size: 20px;
    font-weight: bold;
    height: 52px;
    justify-content: center;
    margin-top: 10px;
    text-transform: uppercase;
    userSelect: none;
    width: 80%;
  `;
  return button;
};

const readyStateCheckInterval = setInterval(() => {
  const el = document
    .querySelector('game-app')
    .shadowRoot.querySelector('game-theme-manager')
    .querySelector('game-modal game-stats');

  if (el && !el.querySelector('#extended-share-button')) {
    clearInterval(readyStateCheckInterval);
    const button = buildButton();
    const shareContainer = el.shadowRoot.querySelector('.share');
    shareContainer.style.flexDirection = 'column';

    shareContainer
      .querySelectorAll('#extended-share-button')
      .forEach((el) => el.remove());

    shareContainer.appendChild(button);
  }
}, 1000);
